==> v11.1.2.zip: brewman/brewman/brewman/routers/db_integrity.py <==
from fastapi import APIRouter, Security
from sqlalchemy import delete, desc, distinct, func, over, select
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionFuture
from ..models.attendance import Attendance
from ..schemas.settings import AttendanceCount
from ..schemas.user import UserToken


router = APIRouter()


@router.post("", response_model=AttendanceCount)
def post_check_db(user: UserToken = Security(get_user)) -> AttendanceCount:
    with SessionFuture() as db:
        duplicate_attendances = get_duplicate_attendances(db)
        if duplicate_attendances > 0:
            fix_duplicate_attendances(db)
        db.commit()
        return AttendanceCount(attendance_count=duplicate_attendances)


def get_duplicate_attendances(db: Session) -> int:
    sub_query = select(  # type: ignore[var-annotated]
        over(
            distinct(func.first_value(Attendance.id)),  # type: ignore[arg-type]
            partition_by=[Attendance.employee_id, Attendance.date_],
        )
    ).where(
        Attendance.is_valid == True  # noqa: E712
    )
    query: int = db.execute(
        select(func.count(Attendance.id)).where(
            ~Attendance.id.in_(sub_query), Attendance.is_valid == True  # noqa: E712
        )
    ).scalar_one()
    return query


def fix_duplicate_attendances(db: Session) -> None:
    sub = (
        select(
            over(
                distinct(func.first_value(Attendance.id)),  # type: ignore[arg-type]
                partition_by=[Attendance.employee_id, Attendance.date_],
                order_by=desc(Attendance.creation_date),
            )
        )
        .where(Attendance.is_valid == True)  # noqa: E712
        .subquery()
    )
    db.execute(delete(Attendance).where(~Attendance.id.in_(sub), Attendance.is_valid == True))  # type: ignore[arg-type]  # noqa: E712

==> v11.6.0.zip: brewman/brewman/brewman/routers/db_integrity.py <==
from fastapi import APIRouter, Security
from sqlalchemy import delete, desc, distinct, func, over, select
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionFuture
from ..models.attendance import Attendance
from ..schemas.settings import AttendanceCount
from ..schemas.user import UserToken


router = APIRouter()


@router.post("", response_model=AttendanceCount)
def post_check_db(user: UserToken = Security(get_user)) -> AttendanceCount:
    with SessionFuture() as db:
        duplicate_attendances = get_duplicate_attendances(db)
        if duplicate_attendances > 0:
            fix_duplicate_attendances(db)
        db.commit()
        return AttendanceCount(attendance_count=duplicate_attendances)


def get_duplicate_attendances(db: Session) -> int:
    sub_query = select(  # type: ignore[var-annotated]
        over(
            distinct(func.first_value(Attendance.id)),  # type: ignore[arg-type]
            partition_by=[Attendance.employee_id, Attendance.date_],
        )
    ).where(
        Attendance.is_valid == True  # noqa: E712
    )
    query: int = db.execute(
        select(func.count(Attendance.id)).where(
            ~Attendance.id.in_(sub_query),
            Attendance.is_valid == True,  # noqa: E712
        )
    ).scalar_one()
    return query


def fix_duplicate_attendances(db: Session) -> None:
    sub = (
        select(
            over(
                distinct(func.first_value(Attendance.id)),  # type: ignore[arg-type]
                partition_by=[Attendance.employee_id, Attendance.date_],
                order_by=desc(Attendance.creation_date),
            )
        )
        .where(Attendance.is_valid == True)  # noqa: E712
        .subquery()
    )
    db.execute(delete(Attendance).where(~Attendance.id.in_(sub), Attendance.is_valid == True))  # type: ignore[arg-type]  # noqa: E712
